import logging
import sys

from PyQt5.QtWidgets import QApplication

from Rare.Styles import dark
from Rare.utils import RareConfig

from legendary.core import LegendaryCore

from Rare.Launch import LaunchDialog
from Rare.Login import LoginWindow
from Rare.MainWindow import MainWindow

logging.basicConfig(
    format='[%(name)s] %(levelname)s: %(message)s',
    level=logging.INFO
)
logger = logging.getLogger("Rare")
core = LegendaryCore()


def main():
    app = QApplication(sys.argv)
    if RareConfig.THEME == "dark":
        app.setStyleSheet(dark)
    logger.info("Try if you are logged in")
    try:
        if core.login():

            logger.info("You are logged in")
        else:
            logger.error("Login Failed")
            main()

    except ValueError:
        logger.info("You ar not logged in. Open Login Window")
        login_window = LoginWindow(core)
        if not login_window.login():
            return
    launch_dialog = LaunchDialog(core)
    if RareConfig.THEME == "dark":
        launch_dialog.setStyleSheet(dark)
    launch_dialog.exec_()
    mainwindow = MainWindow(core)
    app.exec_()


if __name__ == '__main__':
    main()



from logging import getLogger

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QVBoxLayout, QLabel, QPushButton, QScrollArea

from Rare.Tabs.Settings.Rare import RareSettingsForm
from Rare.Tabs.Settings.LegendarySettingsForm import SettingsForm
from Rare.utils.legendaryUtils import get_name
from legendary.core import LegendaryCore

logger = getLogger("Settings")


class SettingsTab(QScrollArea):
    def __init__(self, parent, core: LegendaryCore):
        super(SettingsTab, self).__init__(parent=parent)
        self.core = core
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        # Settings
        self.layout = QVBoxLayout()
        self.layout.addWidget(QLabel("<h1>Rare Settings</h1>"))
        self.logged_in_as = QLabel(f"Logged in as {get_name()}")
        self.legendary_form =SettingsForm()
        self.rare_form = RareSettingsForm()

        self.logout_button = QPushButton("Logout")
        self.logout_button.clicked.connect(self.logout)
        self.layout.addWidget(self.logged_in_as)

        self.layout.addWidget(self.rare_form)
        self.layout.addWidget(self.legendary_form)
        self.layout.addStretch(1)
        self.layout.addWidget(self.logout_button)

        self.info_label = QLabel("<h2>Credits</h2>")
        self.infotext = QLabel("Developers : Dummerle, CommandMC\nLegendary Dev: Rodney\nLicense: GPL v.3")

        self.layout.addWidget(self.info_label)
        self.layout.addWidget(self.infotext)

        self.setLayout(self.layout)

    def logout(self):
        self.core.lgd.invalidate_userdata()
        exit(0)



from PyQt5.QtWidgets import QMainWindow, QTabWidget, QWidget

from Rare.Tabs import SettingsTab, UpdateTab, GameListInstalled,GameListUninstalled, BrowserTab


class MainWindow(QMainWindow):

    def __init__(self, core):
        super().__init__()
        self.setWindowTitle("Rare - GUI for legendary-gl")
        self.setGeometry(0, 0, 1200, 900)
        self.setCentralWidget(TabWidget(self, core))
        self.show()


class TabWidget(QTabWidget):

    def __init__(self, parent, core):
        super(QWidget, self).__init__(parent)

        self.game_list = GameListInstalled(self, core)
        self.addTab(self.game_list, "Games")

        self.uninstalled_games = GameListUninstalled(self, core)
        self.addTab(self.uninstalled_games, "Install Games")

        self.update_tab = UpdateTab(self, core)
        self.addTab(self.update_tab, "Updates")

        self.browser = BrowserTab(self)
        self.addTab(self.browser, "Store")

        self.settings = SettingsTab(self, core)
        self.addTab(self.settings, "Settings")



import os
from logging import getLogger

from PyQt5.QtWidgets import QTableWidget, QTableWidgetItem, QFormLayout, QGroupBox, QLineEdit, QPushButton, \
    QLabel, QCheckBox

from Rare.utils import legendaryConfig


class SettingsForm(QGroupBox):
    config: dict

    def __init__(self, app_name: str):
        self.app_name = app_name
        self.logger = getLogger(f"{app_name} Settings")
        config: dict
        super(SettingsForm, self).__init__(f'Settings for Game {self.app_name}')
        self.config = legendaryConfig.get_config()
        if not self.config.get(self.app_name):
            self.config[self.app_name] = {}
        if not self.config[self.app_name].get("wine_executable"):
            self.config[self.app_name]["wine_executable"] = ""
        if not self.config[self.app_name].get("wine_prefix"):
            self.config[self.app_name]["wine_prefix"] = ""
        if not self.config[self.app_name].get("language"):
            self.config[self.app_name]["language"] = ""
        # if not self.config["Legendary"].get("offline"):
        #     self.config["Legendary"]["offline"] = ""

        env_vars = self.config.get(f"{self.app_name}.env")
        if env_vars:
            self.table = QTableWidget(len(env_vars), 2)
            for i, label in enumerate(env_vars):
                self.table.setItem(i, 0, QTableWidgetItem(label))
                self.table.setItem(i, 1, QTableWidgetItem(env_vars[label]))

        else:
            self.table = QTableWidget(0, 2)

        self.table.setHorizontalHeaderLabels(["Variable", "Value"])

        self.form = QFormLayout()

        self.game_conf_wine_prefix = QLineEdit(self.config[self.app_name]["wine_prefix"])
        self.game_conf_wine_prefix.setPlaceholderText("Default")
        self.game_conf_wine_exec = QLineEdit(self.config[self.app_name]["wine_executable"])
        self.game_conf_wine_exec.setPlaceholderText("Default")
        self.language = QLineEdit(self.config[self.app_name]["language"])
        self.language.setPlaceholderText("Default")

        # self.offline = QCheckBox(self.config["offline"] == "false")


        self.add_button = QPushButton("Add Environment Variable")
        self.delete_env_var = QPushButton("Delete selected Variable")
        self.delete_env_var.clicked.connect(self.delete_var)
        self.add_button.clicked.connect(self.add_variable)

        if os.name != "nt":
            self.form.addRow(QLabel("Default Wineprefix"), self.game_conf_wine_prefix)
            self.form.addRow(QLabel("Wine executable"), self.game_conf_wine_exec)
        # self.form.addRow(QLabel("Offline"), self.offline)
        self.form.addRow(QLabel("Environment Variables"), self.table)
        self.form.addRow(QLabel("Add Variable"), self.add_button)
        self.form.addRow(QLabel("Delete Variable"), self.delete_env_var)
        self.form.addRow(QLabel("language"), self.language)
        self.submit_button = QPushButton("Update")
        self.submit_button.clicked.connect(self.update_legendary_settings)
        self.form.addRow(self.submit_button)
        self.setLayout(self.form)

    def add_variable(self):
        print("add row")
        self.table.insertRow(self.table.rowCount())
        self.table.setItem(self.table.rowCount(), 0, QTableWidgetItem(""))
        self.table.setItem(self.table.rowCount(), 1, QTableWidgetItem(""))

    def delete_var(self):
        self.table.removeRow(self.table.currentRow())

    def update_legendary_settings(self):
        self.logger.info("Updating Game Settings")

        # Wine exec
        if not self.config.get(self.app_name):
            self.config[self.app_name] = {}
        if self.game_conf_wine_exec.text() != "":
            self.config[self.app_name]["wine_executable"] = self.game_conf_wine_exec.text()
        elif "wine_executable" in self.config[self.app_name]:
            self.config[self.app_name].pop("wine_executable")

        # Wineprefix
        if self.game_conf_wine_prefix.text() != '':
            self.config[self.app_name]["wine_prefix"] = self.game_conf_wine_prefix.text()
            pass
        elif "wine_prefix" in self.config[self.app_name]:
            self.config[self.app_name].pop("wine_prefix")

        # language
        if self.language.text() != "":
            self.config[self.app_name]["language"] = self.language.text()
        elif "language" in self.config[self.app_name]:
            self.config[self.app_name].pop("language")

        # Env vars
        if self.table.rowCount() > 0:
            self.config[f"{self.app_name}.env"] = {}
            for row in range(self.table.rowCount()):
                self.config[f"{self.app_name}.env"][self.table.item(row, 0).text()] = self.table.item(row, 1).text()
        elif f"{self.app_name}.env" in self.config:
            self.config.pop(f"{self.app_name}.env")

        if self.config.get(self.app_name) == {}:
            self.config.pop(self.app_name)
        legendaryConfig.set_config(self.config)
        print(self.config)


import os
from logging import getLogger

from PyQt5.QtGui import QIntValidator
from PyQt5.QtWidgets import QTableWidget, QTableWidgetItem, QFormLayout, QGroupBox, QLineEdit, QPushButton, \
    QLabel

from Rare.utils import legendaryConfig


class SettingsForm(QGroupBox):
    config: dict

    def __init__(self):
        self.logger = getLogger(f"Legendary Settings")
        config: dict
        super(SettingsForm, self).__init__('Legendary Settings')
        self.config = legendaryConfig.get_config()
        if not self.config.get("Legendary"):
            self.config["Legendary"] = {}
        if not self.config["Legendary"].get("wine_executable"):
            self.config["Legendary"]["wine_executable"] = ""
        if not self.config["Legendary"].get("wine_prefix"):
            self.config["Legendary"]["wine_prefix"] = ""
        if not self.config["Legendary"].get("locale"):
            self.config["Legendary"]["locale"] = ""
        if not self.config["Legendary"].get("max_workers"):
            self.config["Legendary"]["max_workers"] = ""
        if not self.config["Legendary"].get("install_dir"):
            self.config["Legendary"]["install_dir"] = ""
        if not self.config["Legendary"].get("max_memory"):
            self.config["Legendary"]["max_memory"] = ""

        env_vars = self.config.get(f"default.env")
        if env_vars:
            self.table = QTableWidget(len(env_vars), 2)
            for i, label in enumerate(env_vars):
                self.table.setItem(i, 0, QTableWidgetItem(label))
                self.table.setItem(i, 1, QTableWidgetItem(env_vars[label]))

        else:
            self.table = QTableWidget(0, 2)

        self.table.setHorizontalHeaderLabels(["Variable", "Value"])

        self.form = QFormLayout()

        self.lgd_conf_wine_prefix = QLineEdit(self.config["Legendary"]["wine_prefix"])
        self.lgd_conf_wine_prefix.setPlaceholderText("Default")
        self.lgd_conf_wine_exec = QLineEdit(self.config["Legendary"]["wine_executable"])
        self.lgd_conf_wine_exec.setPlaceholderText("Default")
        self.lgd_conf_locale = QLineEdit(self.config["Legendary"]["locale"])
        self.lgd_conf_locale.setPlaceholderText("Default")

        only_int = QIntValidator()
        self.max_workers = QLineEdit(self.config["Legendary"]["max_workers"])
        self.max_workers.setPlaceholderText("Default")
        self.max_workers.setValidator(only_int)

        self.default_install_dir = QLineEdit(self.config["Legendary"]["install_dir"])
        self.default_install_dir.setPlaceholderText("Default")

        self.max_mem = QLineEdit(self.config["Legendary"]["max_memory"])
        self.max_mem.setPlaceholderText("Default")
        self.max_mem.setValidator(only_int)

        self.add_button = QPushButton("Add Environment Variable")
        self.delete_env_var = QPushButton("Delete selected Variable")
        self.delete_env_var.clicked.connect(self.delete_var)
        self.add_button.clicked.connect(self.add_variable)

        if os.name != "nt":
            self.form.addRow(QLabel("Default Wineprefix"), self.lgd_conf_wine_prefix)
            self.form.addRow(QLabel("Wine executable"), self.lgd_conf_wine_exec)
        self.form.addRow(QLabel("Max Workers"), self.max_workers)
        self.form.addRow(QLabel("Default install dir"), self.default_install_dir)
        self.form.addRow(QLabel("Max memory"), self.max_mem)
        self.form.addRow(QLabel("Environment Variables"), self.table)
        self.form.addRow(QLabel("Add Variable"), self.add_button)
        self.form.addRow(QLabel("Delete Variable"), self.delete_env_var)
        self.form.addRow(QLabel("Locale"), self.lgd_conf_locale)
        self.submit_button = QPushButton("Update")
        self.submit_button.clicked.connect(self.update_legendary_settings)
        self.form.addRow(self.submit_button)
        self.setLayout(self.form)

    def add_variable(self):
        print("add row")
        self.table.insertRow(self.table.rowCount())
        self.table.setItem(self.table.rowCount(), 0, QTableWidgetItem(""))
        self.table.setItem(self.table.rowCount(), 1, QTableWidgetItem(""))

    def delete_var(self):
        self.table.removeRow(self.table.currentRow())

    def update_legendary_settings(self):
        self.logger.info("Updating Game Settings")

        # Wine exec
        if not self.config.get("Legendary"):
            self.config["Legendary"] = {}
        if self.lgd_conf_wine_exec.text() != "":
            self.config["Legendary"]["wine_executable"] = self.lgd_conf_wine_exec.text()
        elif "wine_executable" in self.config["Legendary"]:
            self.config["Legendary"].pop("wine_executable")

        # Wineprefix
        if self.lgd_conf_wine_prefix.text() != '':
            self.config["Legendary"]["wine_prefix"] = self.lgd_conf_wine_prefix.text()
        elif "wine_prefix" in self.config["Legendary"]:
            self.config["Legendary"].pop("wine_prefix")

        # Locale
        if self.lgd_conf_locale.text() != "":
            self.config["Legendary"]["locale"] = self.lgd_conf_locale.text()
        elif "locale" in self.config["Legendary"]:
            self.config["Legendary"].pop("locale")

        # Max Workers
        if self.max_workers.text() != "":
            self.config["Legendary"]["max_workers"] = self.max_workers.text()
        elif "max_workers" in self.config["Legendary"]:
            self.config["Legendary"].pop("max_workers")

        # default installdir
        if self.default_install_dir.text() != "":
            self.config["Legendary"]["install_dir"] = self.default_install_dir.text()
        elif "install_dir" in self.config["Legendary"]:
            self.config["Legendary"].pop("install_dir")

        # max mem
        if self.max_mem.text() != "":
            self.config["Legendary"]["max_memory"] = self.max_mem.text()
        elif "max_memory" in self.config["Legendary"]:
            self.config["Legendary"].pop("max_memory")

        # Env vars
        if self.table.rowCount() > 0:
            self.config[f"default.env"] = {}
            for row in range(self.table.rowCount()):
                self.config[f"default.env"][self.table.item(row, 0).text()] = self.table.item(row, 1).text()
        elif "default.env" in self.config:
            self.config.pop("default.env")

        if self.config.get("Legendary") == {}:
            self.config.pop("Legendary")
        legendaryConfig.set_config(self.config)
        print(self.config)



import time
from logging import getLogger

from PyQt5.QtCore import QThread, QProcess
from PyQt5.QtWidgets import QVBoxLayout, QProgressBar, QPushButton, QLabel, QWidget, QHBoxLayout

from legendary.core import LegendaryCore
from legendary.models.game import InstalledGame
from multiprocessing import Queue as MPQueue
logger = getLogger("Updates")


class UpdateThread(QThread):
    def __init__(self, dlm):
        super(UpdateThread, self).__init__()
        self.dlm = dlm
        logging_queue = MPQueue(-1)
        self.dlm.logging_queue = logging_queue

    def run(self):
        self.dlm.start()
        self.dlm.join()


class UpdateProc(QProcess):
    def __init__(self):
        super(UpdateProc, self).__init__()


class UpdateWidget(QWidget):

    def __init__(self, game: InstalledGame, core: LegendaryCore):
        super(UpdateWidget, self).__init__()
        self.updating = False
        self.game = game
        self.core = core
        self.layout = QVBoxLayout()
        self.childlayout = QHBoxLayout()
        self.label = QLabel("Update available for " + self.game.title)
        self.button = QPushButton("Update")
        self.button.clicked.connect(self.update_game)

        self.childlayout.addWidget(self.label)
        self.childlayout.addWidget(self.button)
        self.layout.addLayout(self.childlayout)
        self.progress_bar = QProgressBar()
        self.progress_bar.setMaximum(100)
        self.layout.addWidget(self.progress_bar)

        self.setLayout(self.layout)

    def update_game2(self):
        if not self.updating:
            logger.info("Update " + self.game.title)
            game = self.core.get_game(self.game.app_name)
            logger.info("Prepare Download")
            dlm, analysis, manifest = self.core.prepare_download(game=game, base_game=game)

            if not analysis.dl_size:
                logger.info("Game is up to date, sorry")

            logger.info(f"Install size: {round(analysis.install_size / (1024 ** 3),2)} GB")
            logger.info(f"Download size: {round(analysis.dl_size/(1024**3), 2)} GB")
            installed_game = self.core.get_installed_game(self.game.app_name)
            res = self.core.check_installation_conditions(analysis, installed_game, self.core.get_game(self.game.app_name), True)
            if res.failures:
                logger.error("Fail")
            if res.warnings:
                for warn in sorted(res.warnings):
                    logger.warning(warn)
            start_time = time.time()
            self.update_thread = UpdateThread(dlm)
            self.update_thread.finished.connect(self.finished)
            self.update_thread.start()
            self.button.setDisabled(True)
            # TODO Wird 1000% nicht funktionieren

        else:
            logger.info("Terminate process")
            self.thread.kill()
            self.button.setText("Update")
            self.updating = False

    def update_game(self):
        self.proc = QProcess()
        self.proc.start("legendary", ["-y", "update", self.game.app_name])

    def finished(self):
        self.setVisible(False)

    def start(self):
        self.button.setText("Cancel")
        self.updating = True

    def get_update_info(self):
        shit, infos, game = self.core.prepare_download(self.core.get_game(self.game.app_name),
                                                       self.core.get_game(self.game.app_name))

    def dataReady(self):
        bytes = self.thread.readAllStandardOutput()
        byte_list = bytes.split('\n')
        data = []
        for i in byte_list:
            data.append(byte_list)

        text = data[0].decode()

        print(data)


class UpdateTab(QWidget):

    def __init__(self, parent, core: LegendaryCore):
        super(UpdateTab, self).__init__(parent=parent)
        self.core = core
        self.layout = QVBoxLayout()
        update_games = self.get_updates()

        if update_games:
            for game in self.get_updates():
                self.layout.addWidget(UpdateWidget(game, core))
        else:
            self.layout.addWidget(QLabel("No updates available"))
        self.layout.addStretch(1)
        self.setLayout(self.layout)

    def get_updates(self):
        self.core.get_assets(True)
        update_games = []
        games=sorted(self.core.get_installed_list(), key=lambda x: x.title)
        versions={}
        for game in games:
            try:
                versions[game.app_name] = self.core.get_asset(game.app_name).build_version
            except:
                logger.error(f"Metadata for {game.title} is missing")

            if versions[game.app_name] != game.version:
                update_games.append(game)
        return update_games
